/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://public.eu-egee.org/partners/ for details on 
 * the copyright holders.
 * For license conditions see the license file or
 * http://eu-egee.org/license.html
 *
 * Authors: 
 *      Paolo Badino <Paolo.Badino@cern.ch>>
 *      Akos Frohner <Akos.Frohner@cern.ch>
 *      Peter Kunszt <Peter.Kunszt@cern.ch>
 *      Gavin McCance <Gavin.McCance@cern.ch>
 *      Krzysztof Nienartowicz <Krzysztof.Nienartowicz@cern.ch>
 *      Ricardo Rocha <Ricardo.Rocha@cern.ch>
 *      Jozsef Kovacs <smith@lutra.sztaki.hu>
 *      Daniel Rodrigues <Daniel.Rodrigues@cern.ch>
 */
package org.glite.data.catalog.service.composite;

import org.glite.data.catalog.service.*;
import org.glite.data.catalog.service.file.*;
import org.glite.data.catalog.service.meta.*;
import org.glite.data.catalog.service.replica.*;


/**
 * The middleware level API for the composite File and SURLStat catalog operations.
 * <br>
 * This interface provides combined operations over the File and
 * Replica Catalog interfaces as well as all operations of the FileCatalog and ReplicaCatalog
 * interfaces which are then simply forwarded to the corresponding services
 * The additional methods giving added value are those which require coordination between
 * the two catalog services.
 */

public interface FiremanCatalog extends FileCatalog, MetadataBase, ReplicaCatalog {

    /**
     * Creates entries based on LFN and SURL only.
     * If GUID is not provided, it is created by the catalog.
     * Only <i>file</i> items are permitted here.
     * 
     * @param entries A list of {@link org.glite.data.catalog.service.FRCEntry FRCEntry} objects.
     *                If permission not passed, one derived from the parent directory is used.
     *                Otherwise only the passed one is used.
     * 
     * @throws AuthorizationException    No access right to create new entry in the parent directory.
     * @throws ExistsException           The file already exists.
     * @throws NotExistsException        One of the parent path elements does not exists.
     * @throws InvalidArgumentException  Some part of the argument is invalid.
     * @throws InternalException
	 *
	 * <dt>
	 * <br><b>Semantic description of the method:</b>
	 * <br>The following steps are performed for each entry:
	 * <ul>
	 * <li> Check format of path, GUID (if given), SURLS, size, etc;
	 * <br> if any of them is invalid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
	 * <li> Check if parent directory exists;
	 * <br> if directory does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}.
	 * <li> Check if file exists;
	 * <br> if file exists, throws {@link org.glite.data.catalog.service.ExistsException}.
	 * <li> Check WRITE permission on parent directory;
	 * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
	 * <li> Insert LFN into catalog with all accompanying metadata.
	 * </ul>
     */
    public void create(FRCEntry[] entries) throws InternalException, AuthorizationException, ExistsException, NotExistsException, InvalidArgumentException;

    /**
     * List all known SURLs registered to a given LFN.
     *
     * @param lfns The Logical File Name to list SURLs for.
     * @param withPermissions Flag whether to include permissions in the listing
     * @return The list of SURLs.
     * 
     * @throws AuthorizationException    No access right to read LFN.
     * @throws NotExistsException        LFN does not exists.
     * @throws InvalidArgumentException  Some part of the argument is invalid.
     * @throws InternalException
	 *
	 * <dt>
	 * <br><b>Semantic description of the method:</b>
	 * <br>The following steps are performed for each entry:
	 * <ul>
	 * <li> Check format of LFN;
	 * <br> if it is invalid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
	 * <li> Check if LFN exists;
	 * <br> if it does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}.
	 * <li> Check if LFN is a file;
	 * <br> if it is not a file, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
	 * <li> Check LIST permission on lfn;
	 * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
	 * <li> Query replicas from database and add them to the list to be returned.
	 * </ul>
	 * </dt>
     */
    public FRCEntry[] listReplicas(String[] lfns, boolean withPermissions) throws InternalException, AuthorizationException, NotExistsException;

    /**
     * Lists LFN for SURL of interest
     * 
     * @param surls list of surls we want perform LFN lookup
     * @return list of {SURL,LFN} pairs
     * 
     * @throws AuthorizationException   No access right to list the LFN for a given SURL.
     * @throws NotExistsException       The SURL does not exist.
     * @throws InvalidArgumentException Some part of the argument is invalid.
     * @throws InternalException
	 *
	 * <dt>
	 * <br><b>Semantic description of the method:</b>
	 * <br>The following steps are performed for each entry:
	 * <ul>
	 * <li> Check format of SURL;
	 * <br> if it is invalid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
	 * <li> Check if SURL exists;
	 * <br> if it does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}.
	 * <li> Check LIST permission on corresponding LFN;
	 * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
	 * <li> Query for LFN from catalog and add it to the list to be returned.
	 * </ul>
	 * </dt>
     */
    public StringPair[] listLfn(String[] surls) throws InternalException, AuthorizationException, NotExistsException, InvalidArgumentException;

    /**
     * Lists LFNStat for a given LFN.
     * 
     * @param lfns list of LFNs we want LFNStat object for.
     * @return list holding LFNStat for the LFNs.

     * @throws InvalidArgumentException Some part of the argument is invalid.
     * @throws AuthorizationException   No access right get status information.
     * @throws NotExistsException       The LFN does not exist.
     * @throws InternalException
     * 
	 * <dt>
	 * <br><b>Semantic description of the method:</b>
	 * <br>The following steps are performed for each entry:
	 * <ul>
 	 * <li> Check format of LFN;
	 * <br> if it is invalid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
	 * <li> Check if LFN exists;
	 * <br> if it does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}.
	 * <li> Check LIST permission on the LFN;
	 * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
	 * <li> Get relevant LFNStat details from the catalog, add them to the list to be returned.
	 * </ul>
     * 
     */

    public Stat[] getStat(String[] lfns) throws InternalException, AuthorizationException, NotExistsException, InvalidArgumentException;


    /**
     * New mapping creation for {LFN,SURL[]} pairs.
     *
     * @param lfn       LFN to add new SURLs to, has to be a file.
     * @param newSurls  The list of new SURLs to add
     * 
     * @throws AuthorizationException   No right to add new SURLs to an existing replica.
     * @throws ExistsException          The LFN-SURL pair already exists.
     * @throws InvalidArgumentException Some part of the argument is invalid.
     * @throws NotExistsException       The LFN does not exist.
     * @throws InternalException
     * 
	 * <dt>
	 * <br><b>Semantic description of the method:</b>
	 * <br>The following steps are performed:
	 * <ul>
	 * <li> Check format of LFN, SURLs;
	 * <br> if any of them is invalid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
	 * <li> Check if LFN exists;
	 * <br> if it does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}.
	 * <li> Check if LFN is a file;
	 * <br> if it is not, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
	 * <li> Check SET_METADATA permission on the LFN;
	 * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
	 * <li> For each SURLentry, check if it exists already;
	 * <br> if it exists, throws {@link org.glite.data.catalog.service.ExistsException}.
	 * <li> Add new replica info into the catalog.
	 * </ul>
     */ 
    public void addReplica(String lfn, SURLEntry[] newSurls) throws InternalException, AuthorizationException, ExistsException, NotExistsException, InvalidArgumentException;

    /**
     * Removes LFN and all linked data (corresponding guid and replicas) from catalog.
     * 
     * @param lfns LFN for removal, cannot be directories
     * 
     * @throws AuthorizationException   No right to remove the entry from the catalog.
     * @throws InvalidArgumentException Some part of the argument is invalid.
     * @throws NotExistsException       The LFN does not exist.
     * @throws InternalException
     * 
	 * <dt>
	 * <br><b>Semantic description of the method:</b>
	 * <br>The following steps are performed for each entry:
	 * <ul>
	 * <li> Check format of LFN;
	 * <br> if it is invalid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
	 * <li> Check if LFN exists;
	 * <br> if it does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}.
	 * <li> Check if LFN is a file;
	 * <br> if it is not, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
	 * <li> Check REMOVE permission on the LFN;
	 * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
	 * <li> Removes LFN and all linked data (GUID, SURLs, Stats, Permissions) from the catalog.
	 * </ul>
     */
    public void remove(String[] lfns) throws InternalException, AuthorizationException, NotExistsException, InvalidArgumentException;

    /**
     * Removes specified mappings. Each string pair contains a mapping to be removed.
     * 
     * @param lfnSurls list of {LFN, SURL} pair objects.
     * The first string is the LFN, the second is the SURL to be removed.
     * @param removeLfnOnLastReplica tells the catalog whether the LFN should be removed if the last SURL is removed.
     * 
     * @throws AuthorizationException   No right to remove a replica from the given LFN.
     * @throws NotExistsException       Either the LFN does not exist or the given SURL is not a registered replica of the LFN.
     * @throws InvalidArgumentException The LFN or SURL of an entry is malformed.
     * @throws InternalException
     * 
     * <dt>
     * <br><b>Semantic description of the method:</b>
     * <br>The following steps are performed for each entry:
     * <ul>
     * <li> Check if LFN and corresponding SURL are valid;
     * <br> if either one of them is not valid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
     * <li> Check if LFN and corresponding SURL exist;
     * <br> if any of them does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}.
     * <li> Check SET_METADATA permission on LFN;
     * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
     * <li> Check if the SURL in the list is the Master one;
     * <br> if it is the Master and it is not the last replica, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
     * <li> Remove SURL of LFN from the catalog.
     * <li> Check if the removeLfnOnLastReplica is set;
     * <br> if it is set and it was the last replica, then continue, otherwise return.
     * <li> Check REMOVE permission on LFN;
     * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
     * <li> Remove LFN.
     * </ul>
     * </dt>
     */
    public void removeReplica(StringPair[] lfnSurls, boolean removeLfnOnLastReplica) throws InternalException, AuthorizationException, NotExistsException, InvalidArgumentException;

    // -------------------------------
    // -- MetadataCatalog speciality
    // -------------------------------
    /**
     * Associates an existing directory with an existing schema. The semantics of this operation should
     * be that all files inside the given directory inherit and share this schema.
     * @param directoryLfn The lfn of the directory.
     * @param schemaName The name of the schema to be associated with the directory.
     * @throws InternalException
     * @throws AuthorizationException   No right to list entries in the catalog.
     */
    public void associateDirWithSchema(String directoryLfn, String schemaName) throws InternalException;
}
