/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://public.eu-egee.org/partners/ for details on 
 * the copyright holders.
 * For license conditions see the license file or
 * http://eu-egee.org/license.html
 *
 * Authors: 
 *      Paolo Badino <Paolo.Badino@cern.ch>>
 *      Akos Frohner <Akos.Frohner@cern.ch>
 *      Peter Kunszt <Peter.Kunszt@cern.ch>
 *      Gavin McCance <Gavin.McCance@cern.ch>
 *      Krzysztof Nienartowicz <Krzysztof.Nienartowicz@cern.ch>
 *      Ricardo Rocha <Ricardo.Rocha@cern.ch>
 *      Jozsef Kovacs <smith@lutra.sztaki.hu>
 *      Daniel Rodrigues <Daniel.Rodrigues@cern.ch>
 */
package org.glite.data.catalog.service.fas;

import org.glite.data.catalog.service.AuthorizationException;
import org.glite.data.catalog.service.ExistsException;
import org.glite.data.catalog.service.InternalException;
import org.glite.data.catalog.service.InvalidArgumentException;
import org.glite.data.catalog.service.NotExistsException;
import org.glite.data.catalog.service.Perm;
import org.glite.data.catalog.service.PermissionEntry;
import org.glite.data.catalog.service.ServiceBase;

/**
 * The middleware level API for File Access (or Authorization) Service.
 *
 * @stereotype simple-interface
 */

public interface FASBase extends ServiceBase {
    /**
     * Sets full set of permissions {BasicPermission,ACL} for a given item (GUID, LFN or SchemaName).
     * 
     * @param permissions - list of {item, permission} pairs.
     *                      The {@link org.glite.data.catalog.service.PermissionEntry PermissionEntry}
     *                      object takes the item and a list of ACLs.
     *                      This method sets the full permission, replacing all existing ACLs.
     * 
     * @throws AuthorizationException   No access right to update the permissions.
     * @throws NotExistsException       The item does not exist.
     * @throws InvalidArgumentException Some part of the argument is invalid.
     * @throws InternalException
     * 
	 * <dt>
	 * <br><b>Semantic description of the method:</b>
	 * <br>The following steps are performed for each entry:
	 * <ul>
	 * <li> Check format of item (depeds on the context);
	 * <br> if it is invalid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
	 * <li> Check if the item exists;
	 * <br> if it does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}.
	 * <li> Check PERMISSION permission on the item;
	 * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
	 * <li> Updates permission details in the catalog.
	 * </ul>
     */
    public void setPermission(PermissionEntry[] permissions) throws InternalException, AuthorizationException, NotExistsException, InvalidArgumentException;

    /**
     * Retrieves all set permissions for any kind of item (GUID, LFN or SchemaName).
     * 
     * @param items list of items we want permission for
     * @return returns item with corresponding permission
     * 
     * @throws AuthorizationException   No access right to get the permission information.
     * @throws NotExistsException       The item does not exist.
     * @throws InvalidArgumentException Some part of the argument is invalid.
     * @throws InternalException
     * 
	 * <dt>
	 * <br><b>Semantic description of the method:</b>
	 * <br>The following steps are performed for each entry:
	 * <ul>
	 * <li> Check format of item (depends on the context);
	 * <br> if it is invalid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
	 * <li> Check if item exists;
	 * <br> if it does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}.
	 * <li> Check LIST permission on the item;
	 * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
	 * <li> Query permission details from the catalog and add it to the list to be returned.
	 * </ul>
     */
    public PermissionEntry[] getPermission(String[] items) throws InternalException, AuthorizationException, NotExistsException, InvalidArgumentException;
    
    /**
     * Checks if the current user has the required permission bits on the specified items.
     *
     * @param items list of items we want to check permission of 
     * @param perm the permission bits to be checked against
     *
     * @throws AuthorizationException   No access right to check permission information
     * <!--@throws NotExistsException   The item does not exist
     * @throws InvalidArgumentException Some part of the argument is invalid.-->
     * @throws InternalException
     * 
	* <dt>
	* <br><b>Semantic description of the method:</b>
	* <br>The following steps are performed for each item entry:
	* <ul> 
	* <!--<li> Check format of item (depends on the context)
	* <br> If it is invalid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
	* <li> Check if item exists
	* <br> If it does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}.-->
	* <li> Check PERMISSION permission on the item;
        * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
	* <li> Check required permission bits on the item;
	* <br> if any of the permission bits is not verified, throws {@link org.glite.data.catalog.service.AuthorizationException}. 
	*	
     */
    public void checkPermission(String[] items, Perm perm) throws InternalException, AuthorizationException;
}

