/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://eu-egee.org/partners/ for details on the copyright holders.
 * For license conditions see the license file or http://eu-egee.org/license.html
 * 
 * Authors: 
 *      Paolo Badino <Paolo.Badino@cern.ch>>
 *      Akos Frohner <Akos.Frohner@cern.ch>
 *      Peter Kunszt <Peter.Kunszt@cern.ch>
 *      Gavin McCance <Gavin.McCance@cern.ch>
 *      Krzysztof Nienartowicz <Krzysztof.Nienartowicz@cern.ch>
 *      Ricardo Rocha <Ricardo.Rocha@cern.ch>
 *      Jozsef Kovacs <smith@lutra.sztaki.hu>
 *      Daniel Rodrigues <Daniel.Rodrigues@cern.ch>
 */

package org.glite.data.catalog.service.meta;

import org.glite.data.catalog.service.*;

/**
 * The middleware level API for handling attributes of individual items/entries
 * in the Metadata Catalog component. This is the interface that the
 * service exposes to the other middleware components, which is not
 * necessarily the final API seen by the user.
 */
public interface MetadataBase extends ServiceBase {
    /**
     * Sets the values for a group of attributes of an item/entry.
     * <p>
     * The item is the string that uniquely identifies the entry in the catalog. It can be a GUID,
     * an LFN, or any other type of identifier.
     * The attributes parameter is a list (array) of {@link org.glite.data.catalog.service.Attribute}
     * objects, each containing the necessary information to set the new values.
     * </p><p>
     * An implementation can choose to offer pure POSIX xattrs semantics, allowing
     * on the fly creation of attributes when they do not yet exist. The type field
     * in each {@link org.glite.data.catalog.service.Attribute} object should be filled in this case.
     * Or it may not support this feature, and the type field can be left null.
     * </p>
     *
     * @param item The item/entry in the catalog where the values should be set.
     * @param attributes A list of Attribute objects containing the name/type/value to be set.
     * Both the name and value fields in the Attribute object MUST be given.
     *
     * @throws AuthorizationException No access right to update values of attributes.
     * @throws NotExistsException The entry/item or one of the attributes specified does not exist.
     * @throws InvalidArgumentException One of the name/type pairs given for an attribute is invalid
     * - case of on-the-fly creation of attributes, where the type given is unsupported by the backend.
     * Or one of the attribute's values is invalid - while trying to update values.
     * @throws InternalException Any other error on the server side (i.e. database down).
     *
     * <dt>
         * <br/><b>Semantic description of the method:</b>
         * <br/>First steps performed:
         * <ul>
         * <li>Check if the item/entry exists in the catalog:
         * <br/>if it does not, throw {@link org.glite.data.catalog.service.NotExistsException}.</li>
         * <li>Check if client is allowed to change metadata for item/entry:
         * <br/>if it is not, throw {@link org.glite.data.catalog.service.AuthorizationException}.</li>
         * </ul>
         * The following additional steps are performed for each attribute:
         * <ul>
         * <li>Check if attribute exists in the catalog. If it does not, what happens next depends on the
         * implementation:
         * <br/>if on-the-fly creation of attributes is supported, an additional step for checking if the
         * attribute's type is supported by the backend should be done. If it is not, throw
         * {@link org.glite.data.catalog.service.InvalidArgumentException}.
         * <br/>if this feature is not supported, throw
         * {@link org.glite.data.catalog.service.NotExistsException}.</li>
         * <li>Check validity of value given for the attribute (depends on the type):
         * <br/>if invalid, throw {@link org.glite.data.catalog.service.InvalidArgumentException}.</li>
         * </ul>
         * Set the values of the attributes.
         *
         * @see org.glite.data.catalog.service.Attribute
         * @see org.glite.data.catalog.service.meta
     */
    public void setAttributes(String item, Attribute[] attributes) throws AuthorizationException, NotExistsException, InvalidArgumentException, InternalException;

    /**
     * Clears the values for a group of attributes.
     *
     * <p>The item is the unique identifier of items/entries in the catalog. It can be a GUID, an LFN,
     * or any other kind of string identifier. The attributeNames is a list of the names of the attributes
     * inside the catalog.</p>
     *
     * @param item The item/entry where attributes should be cleared.
     * @param attributeNames An array containing the names of the attributes to be cleared.
     *
     * @throws AuthorizationException No access right to update values of attributes.
     * @throws NotExistsException The item/entry or one of the attributes specified does not exist.
     * @throws InternalException Any other error on the server side (i.e. database down).
     * <dt>
         * <br/><b>Semantic description of the method:</b>
         * <br/>First steps performed:
         * <ul>
         * <li>Check if the entry/item exists in the catalog:
         * <br/>if it does not, throw {@link org.glite.data.catalog.service.NotExistsException}.</li>
         * <li>Check if client is allowed to change metadata for item/entry:
         * <br/>if it is not, throw {@link org.glite.data.catalog.service.AuthorizationException}.</li>
         * </ul>
         * The following steps are performed for each attribute name:
         * <ul>
         * <li>Check if there is an attribute in the catalog with the given name:
         * <br/>if there is none, throw {@link org.glite.data.catalog.service.NotExistsException}.</li>
         * </ul>
         * Clear the values of the attributes.
         *
         * @see org.glite.data.catalog.service.meta
    */
    public void clearAttributes(String item, String[] attributeNames) throws AuthorizationException, NotExistsException, InternalException;

    /**
     * Gets a group of attributes of a given item/entry.
     *
     * <p>The item is the unique identifier of items/entries in the catalog. It can be a GUID, an LFN,
     * or any other kind of string identifier. The attributeNames is a list of the names of the attributes
     * inside the catalog.</p>
     *
     * @param item The item/entry in the catalog for which the given attributes should be returned.
     * @param attributeNames The names of the attributes to return the values.
     *
     * @return A list of {@link org.glite.data.catalog.service.Attribute} objects containing the
     * requested values. Each {@link org.glite.data.catalog.service.Attribute} object MUST
     * contain the name and value filled in, and MAY contain the type filled in - but not necessarily.
     * Returns null if there is no attribute in the catalog with any of the given names.
     *
     * @throws AuthorizationException No access right to access attributes for this item.
     * @throws NotExistsException The item/entry or one of the attributes specified does not exist.
     * @throws InternalException Any other error on the server side (i.e. database down).
     * <dt>
         * <br/><b>Semantic description of the method:</b>
         * <br/>First steps performed:
         * <ul>
         * <li>Check if the item/entry exists in the catalog:
         * <br/>if it does not, throw {@link org.glite.data.catalog.service.NotExistsException}.</li>
         * <li>Check if client is allowed to access metadata for item/entry:
         * <br/>if it is not, throw {@link org.glite.data.catalog.service.AuthorizationException}.</li>
         * </ul>
         * The following steps are performed for each attribute name requested and are
         * implementation dependent:
         * <ul>
         * <li>If an implementation decided to ignore attribute names for which there is no corresponding
         * attribute in the catalog:
         * <br/>no exception is thrown when one is found.</li>
         * <li>If an implementation restricts queries for existing attributes only:
         * <br/>throw {@link org.glite.data.catalog.service.NotExistsException}.</li>
         * <br/>There is no inconsistence here, as the method returns Attribute objects, which MUST
         * have both name and value filled. The index in the list of the attribute names given and the one
         * in the list of {@link org.glite.data.catalog.service.Attribute} objects returned need not be the
         * same.
         * <br/>The reason for having different semantics is to allow implementation to do one-shot
         * queries in the backend, where the result is either all values for the attributes or an
         * error if at least one of the attributes does not exist.
         * </ul>
         * Return the attributes requested.
         *
         * @see org.glite.data.catalog.service.Attribute
         * @see org.glite.data.catalog.service.meta
     */
    public Attribute[] getAttributes(String item, String[] attributeNames) throws AuthorizationException, NotExistsException, InternalException;

    /**
     * List all attributes associated with an entry.
     *
     * <p>The item passed in the request is the unique identifier of the entry in the catalog.</p>
     *
     * <p>Although the semantics of the method involve returning the names and types of the attributes
     * inside the catalog associated with a given entry, an implementation MAY decide to include the
     * values of the attributes inside each {@link org.glite.data.catalog.service.Attribute} object in the
     * returned list.</p>
     *
     * <p>All attributes associated with the entry MUST be returned - including the ones with no
     * value set.</p>
     *
     * @param item The item/entry to list the attributes.
     *
     * @return An array of {@link org.glite.data.catalog.service.Attribute} objects where the name and
     * the type MUST be filled. Optionally, the value MAY also be filled, allowing a behaviour similar
     * to a single-shot getAllAttributes operation - that does not exist in this interface.
     * Returns null if the item/entry has no attributes.
     *
     * @throws AuthorizationException No access right to access attributes for this entry/item.
     * @throws NotExistsException The item/entry specified does not exist.
     * @throws InternalException Any other error on the server side (i.e. database down).
     * <dt>
         * <br/><b>Semantic description of the method:</b>
         * <br/>The following steps are performed:
         * <ul>
         * <li>Check if the item/entry exists in the catalog:
         * <br/>if it does not, throw {@link org.glite.data.catalog.service.NotExistsException}.</li>
         * <li>Check if client is allowed to get metadata for item/entry:
         * <br/>if it is not, throw {@link org.glite.data.catalog.service.AuthorizationException}.</li>
         * </ul>
         * Return the attributes.
         *
         * @see org.glite.data.catalog.service.Attribute
         * @see org.glite.data.catalog.service.meta
     */
    public Attribute[] listAttributes(String item) throws AuthorizationException, NotExistsException, InternalException;

    // -------------------------------
    // -- GENERIC QUERY OPERATIONS
    // -------------------------------

    /**
     * Performs a generic query on the catalog, returning the corresponding item's identifiers.
     *
     * <p>The query is simply a string that has to be understood by the service. For example, if the
     * catalog has a backend talking SQL, then this string should be a valid SQL query. The limit is the
     * number of records to be returned in a single request. The offset is the number of records that
     * should be ignored from the beginning of the whole resultset of the query.</p>
     *
     * <p>The method returns the identifiers of all items corresponding to the given query. These identifiers
     * are strings, each being a GUID, an LFN, or any other unique string identifier. The number of these
     * items returned in each call to the service can be:
     * <br/>the <em>limit provided in the query</em>, if the number of items corresponding to the query
     * minus the offset is bigger than this limit.
     * <br/>the <em>number of items left</em>, when the total number of items corresponding to the query
     * minus the offset is smaller than the limit provided.
     * </p>
     *
     * <p>The service also enforces its own limits, so the maximum number of items returned in each single
     * call can be checked by calling {@link #getServiceMetadata} and asking for the <em>query limit</em>
     * parameter.</p>
     *
     * <p>Finally, if the service has the ability to handle schemas, the query string MUST contain the
     * scope of the query. In example, for a SQL backend, where the schemas are mapped to tables in the
     * database, the scope of the query is defined in the FROM part of the SQL query.</p>
     *
     * @param query The query itself, which is simply a string that the catalog understands (SQL query,
     *					XPATH query, ...).
     * @param type The type of query presented in the query string. This has to correspond to one of the query
     * 					types the catalog understands, and will typically be 'SQL', 'XPATH', . It
     * 					is up to the implementation to define which query types it supports.
     * @param limit The maximum number of items to be returned. Should always be above or equal to 0 and
     *					below the service limit. A value of 0 means no limit, which in practice means 
     *					only the system limit should be enforced.
     * @param offset The offset for the items returned, used to iterate between results when doing
     * 					repetitive calls with the same query.
     * @return	The string identifiers of all items corresponding to the given query.
     * 			null if there are no items to be returned. This happens when the offset is above the total
     *				number of items corresponding to the query, and is the condition for checking the end of
     *				the iteration through the results.
     *
     * @throws AuthorizationException   No access right to list the entries.
     * @throws InvalidArgumentException The query string given is invalid, or the query type is invalid,
     * 					or the limit given is invalid (below 0 or above the system limit),
     * 					or the offset is invalid (below 0).
     * @throws InternalException 	Any other error on the server side (i.e. database down).
     *
     * <dt>
         * <br/><b>Semantic description of the method:</b>
         * <br/>The following steps are performed:
         * <ul>
         * <li>Check if the client is allowed to perform queries in the catalog:
         * <br/>if it is not, throw {@link org.glite.data.catalog.service.AuthorizationException}.</li>
	 * <li>Check if the given query type is supported by the catalog:
	 * <br/>if it is not, throw {@link org.glite.data.catalog.service.InvalidArgumentException}.</li>
         * <li>Check if the proposed limit for number of items to return is within the service limits:
         * <br/>if it is not, throw {@link org.glite.data.catalog.service.InvalidArgumentException}.</li>
         * <li>Check if the proposed offset is valid in the system:
         * <br/>if it is not, throw {@link org.glite.data.catalog.service.InvalidArgumentException}.</li>
         * <li>Check if the given query is valid:
         * <br/>if it is not, throw {@link org.glite.data.catalog.service.InvalidArgumentException}.</li>
         * <li>Perform the query and return the corresponding item identifiers.</li>
         *
         * @see org.glite.data.catalog.service.meta
         * @see #getServiceMetadata
     */
    public String[] query(String query, String type, int limit, int offset) throws InternalException, AuthorizationException, InvalidArgumentException;
}
