/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://public.eu-egee.org/partners/ for details on 
 * the copyright holders.
 * For license conditions see the license file or
 * http://eu-egee.org/license.html
 *
 * Authors: 
 *      Akos Frohner <Akos.Frohner@cern.ch>
 *      Peter Kunszt <Peter.Kunszt@cern.ch>
 *      Gavin McCance <Gavin.McCance@cern.ch>
 *      Krzysztof Nienartowicz <Krzysztof.Nienartowicz@cern.ch>
 */
package org.glite.data.catalog.service.replica;

import org.glite.data.catalog.service.*;
import org.glite.data.catalog.service.fas.FASBase;

/**
 * The middleware level API for the SURLStat Catalog component.
 * This is the interface that this service exposes
 * to the other middleware components, not directly to the user.
 * <br>
 * <p><b>Invariant for RCEntry</b>
 * <ul>
 * <li> valid GUID
 * <li> list of {@link org.glite.data.catalog.service.SURLEntry SURLEntry}s (can be empty)
 * <li> valid {@link org.glite.data.catalog.service.GUIDStat GUIDStat}
 * <li> there is one and only one master replica set among the SURL entries
 * </ul>
 * </p>
 *
 * @stereotype simple-interface
 */
public interface ReplicaCatalog extends FASBase {
    
    //-------------------------------------
    //methods that write/update the catalog
    //-------------------------------------
    
    /**
     * Creates a set of new mappings in the Replica catalog. 
     * The {@link org.glite.data.catalog.service.RCEntry RCEntry}
     * object contains the GUID, its stat and a list of 
     * {@link org.glite.data.catalog.service.SURLEntry SURLEntry}
     * objects giving all information about the SURLs associated with the given GUID.
     * <br>
     * Permissions are set in the GUIDStat object of the entry. If they are
     * kept empty, defaults are taken.
     * 
     * @param entries list of mapping entities ({@link org.glite.data.catalog.service.RCEntry RCEntry})
     * 
     * @throws AuthorizationException   No right to add new entries.
     * @throws ExistsException          A SURL or GUID already exists in the catalog.
     * @throws InvalidArgumentException     The status information in the entry is invalid.
     * @throws InternalException
     *
     * <dt>
	 * <br><b>Semantic description of the method:</b>
	 * <br>The following steps are performed for each entry:
	 * <ul>
	 * <li> Checks if RCEntry[] is valid
	 * <br> if it is not valid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}
 	 * <li> Check if GUID already exists
	 * <br> if it exists, throws {@link org.glite.data.catalog.service.ExistsException}.
         * <li> Check if SURL's already exist
         * <br> if any exists, throws {@link org.glite.data.catalog.service.ExistsException}.
	 * <li> Check WRITE permission on defaults (see <A href="../../../../../../overview-summary.html#defaultPermission">Virtual Directory Tree of Permissions</A>);
	 * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
	 * <li> Create default GUIDStat, if there is none.
  	 * <li> Insert GUID, Permission and SURLStat details into the catalog.
	 * </ul>
	 *
     */
    public void createGuid(RCEntry[] entries) throws InternalException, AuthorizationException, ExistsException, InvalidArgumentException;


    /**
     * Adds a list of new SURLs for existing entry in the catalog, keyed on the GUID.
     * 
     * @param guid      existing GUID we add SURLS for
     * @param newSurls  list of new SurlStats for an already existing GUID
     * 
     * @throws AuthorizationException   No right to add new SURLs to an existing replica.
     * @throws ExistsException          The GUID-SURL pair already exists.
     * @throws InvalidArgumentException The status information for the new SURL is invalid.
     * @throws InternalException
	 *
	 * <dt>
	 * <br><b>Semantic description of the method:</b>
	 * <br>The following steps are performed:
	 * <ul>
	 * <li> Checks if GUID and SURL'S are valid
	 * <br> if it is not valid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}
	 * <li> Check if GUID exist;
	 * <br> if it does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}.
	 * <li> Check SET_METADATA permission on GUID;
	 * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
	 * <li> For each SURL entries, check if it exists;
	 * <br> if it exists, throws {@link org.glite.data.catalog.service.ExistsException}.
	 * <li> Check if there is maximum one master replica among the new SURLs;
	 * <br> if there is more, throws {@link org.glite.data.catalog.service.InvalidArgumentException}
	 * <li> Update catalog by adding surls to GUID in the catalog.
	 * <br> Master replica setting overrides the setting of the existing SURLs in the catalog.
	 * </ul>
	 * </dt>
     */
    public void addGuidReplica(String guid, SURLEntry[] newSurls) throws InternalException, AuthorizationException, ExistsException, InvalidArgumentException;

    /**
     * Bulk method to update GUID stat objects.
     * @param guids     list of GUIDs to update
     * @param stats     list of GUIDStat containing the update
     * 
     * @throws AuthorizationException   No right to update the replica status information.
     * @throws InvalidArgumentException     The status information for the GUID is invalid.
     * @throws NotExistsException       The GUID does not exists.
     * @throws InternalException
     * 
     * <dt>
	 * <br><b>Semantic description of the method:</b>
	 * <br>The following steps are performed for each Entry:
	 * <ul>
	 * <li> Checks if GUIDStat is valid
	 * <br> if it is not valid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}
 	 * <li> Check if GUID exists;
	 * <br> if it does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}
	 * <li> Check SET_METADATA permissions for the GUID;
	 * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
	 * <li> Update GUIDStat in the catalog; <i>creationTime</i> is ignored (not updated in the catalog).
	 * </ul>
     */
    public void updateGuidStat(String[] guids, GUIDStat[] stats) throws InternalException, AuthorizationException, InvalidArgumentException, NotExistsException;

    /**
     * Sets SURLStat objects on a given GUID.
     * 
     * @param guid GUID to set SURLStat for
     * @param surlStats generic properties of a lsit of SURLs to be set
     * 
     * @throws AuthorizationException   No right to update the replica status information.
     * @throws InvalidArgumentException     The status information for the SURL is invalid.
     * @throws NotExistsException       The GUID or a SURL does not exists.
     * @throws InternalException
     * 
     * <dt>
	 * <br><b>Semantic description of the method:</b>
	 * <br>The following steps are performed:
	 * <ul>
	 * <li> Check if GUID exists;
	 * <br> if it does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}
	 * <li> Check SET_METADATA permissions for the GUID;
	 * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
	 * <li> For each SURL entry:
	 * <ul>	
	 * 		<li> Checks if SURLEntry is valid
	 *		<br> if it is not valid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}
	 * 		<li> Check if SURL exists;
     * 		<br> if it does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}
     * 		<li> Update SURLStat in the catalog.
	 * </ul>
	 * </ul>
     */
    public void updateSurlStat(String guid, SURLEntry[] surlStats) throws InternalException, AuthorizationException, InvalidArgumentException, NotExistsException;

    /**
     * Sets one of existing SURLs as <i>master</i> for a given guid.
     * 
     * @param guidSurls list of {@link org.glite.data.catalog.service.StringPair StringPair} objects. 
     * The first string is the GUID, the second the master SURL to be set.
     * 
     * @throws AuthorizationException   No right to update the replica status information.
     * @throws NotExistsException       Either the GUID does not exist or the given SURL is not a registered replica of the GUID.
     * @throws InvalidArgumentException The GUID or SURL of an entry is malformed.
     * @throws InternalException
	 *
	 * <dt>
	 * <br><b>Semantic description of the method:</b>
	 * <br>The following steps are performed for each entry:
	 * <ul>
	 * <li> Check if GUID and corresponding SURL exist;
	 * <br> if either one of them does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}.
         * <li> Check if GUID and corresponding SURL are valid;
         * <br> if either one of them is not valid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}
	 * <li> Check SET_METADATA permission on GUID;
	 * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
	 * <li> Update catalog, set SURL to be the master replica of the given GUID.
	 * </ul>
	 * </dt>
     */
    public void setMasterReplica(StringPair[] guidSurls) throws InternalException, AuthorizationException, NotExistsException, InvalidArgumentException;

    /**
     * Set the status flags for the given GUID.
     * @param guids GUIDs to set status for.
     * @param status The status flag to set or cleared.
     * @param mask The mask for the status bits to be set or cleared. 
     * 
     * @throws AuthorizationException   No right to update the replica status information.
     * @throws InvalidArgumentException The status information for the SURL is invalid.
     * @throws NotExistsException       The GUID to update does not exist.
     * @throws InternalException
     * 
     * <dt>
	 * <br><b>Semantic description of the method:</b>
	 * <br>The following steps are performed for each Entry:
	 * <ul>
     	 * <li> Check if GUID is valid;
     	 * <br> if it is not valid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
	 * <li> Check if GUID exists;
	 * <br> if it does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}.
	 * <li> Check SET_METADATA permissions for the GUID;
	 * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
	 * <li> Get current Status from the catalog.
	 * <li> Mask status bits.
	 * <li> Update GUID Status in the catalog. 
	 * </ul>
	 */
	public void updateStatus(String[] guids, int status, int mask) throws InternalException, AuthorizationException, InvalidArgumentException, NotExistsException;

    /**
     * Removes specified mappings. Each string pair contains a mapping to be removed.
     * 
     * @param guidSurls list of {@link org.glite.data.catalog.service.StringPair StringPair} objects. 
     * The first string is the GUID, the second is the SURL to be removed.
     * @param removeGuidOnLastReplica tells the catalog whether the GUID should be removed if the last SURL is removed.
     * 
     * @throws AuthorizationException   No right to remove a replica from the given GUID.
     * @throws NotExistsException       Either the GUID does not exist or the given SURL is not a registered replica of the GUID.
     * @throws InvalidArgumentException The GUID or SURL of an entry is malformed.
     * @throws InternalException
     * 
     * <dt>
     * <br><b>Semantic description of the method:</b>
     * <br>The following steps are performed for each entry:
     * <ul>
     * <li> Check if GUID and corresponding SURL are valid;
     * <br> if either one of them is not valid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
     * <li> Check if GUID and corresponding SURL exist;
     * <br> if any of them does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}.
     * <li> Check SET_METADATA permission on GUID;
     * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
     * <li> Check if the SURL in the list is the Master one;
     * <br> if it is the Master and it is not the last replica, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
     * <li> Remove SURL of GUID from the catalog.
     * <li> Check if the removeGuidOnLastReplica is set;
     * <br> if it is set and it was the last replica, then continue, otherwise return.
     * <li> Check REMOVE permission on GUID;
     * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
     * <li> Remove GUID.
     * </ul>
     * </dt>
     */
    public void removeGuidReplica(StringPair[] guidSurls, boolean removeGuidOnLastReplica) throws InternalException, AuthorizationException, NotExistsException, InvalidArgumentException;

    /**
     * Removes GUID with all existing mappings from the catalog.
     * 
     * @param guids     The replica entries to be removed.
     * 
     * @throws AuthorizationException   No right to remove the replica entry from the catalog.
     * @throws NotExistsException       The GUID does not exist.
     * @throws InternalException
     * 
     * <dt>
     * <br><b>Semantic description of the method:</b>
     * <br>The following steps are performed for each entry:
     * <ul>
     * <li> Check the format of the GUID;
     * <br> if it is invalid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
     * <li> Check if GUID exists;
     * <br> if it does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}.
     * <li> Check REMOVE permission for the GUID;
     * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
     * <li> Remove GUID entry from the catalog;
     * </ul>
     * 
     */
    public void removeGuid(String[] guids) throws InternalException, AuthorizationException, NotExistsException, InvalidArgumentException;


    /**
     * Default permissions for user. Defaults to be used for newly created GUIDs
     * where the GUIDStat object does not contain any permissions. If the user
     * does not have default user permissions set, the global permissions are
     * used, see {@link #setDefaultGlobalPermission setDefaultGlobalPermission}.
     * Sets permission policy for a user used while creating new RCEntries
     * with no permissions specified.
     * <br>
     * For detailed description of the default permimssions semantics see <A href="../../../../../../overview-summary.html#defaultPermission">Virtual Directory Tree of Permissions</A>.
     *
     * 
     * @param defaultPermission - permission object to be used while no permissions are specified
     * @see  <A href="../../../../../../overview-summary.html#defaultPermission">Virtual Directory Tree of Permissions</A>
     * 
     * @throws AuthorizationException   No access right to update the default user permissions.
     * @throws InvalidArgumentException The permission settings are invalid.
     * @throws InternalException
     *
     * <dt>
     * <br><b>Semantic description of the method:</b>
     * <br>The following steps are performed:
     * <ul>
     * <li> Check the validity of permission settings;
     * <br> if it is invalid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
     * <li> Check WRITE permission on default user permission entry;
     * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
     * <li> Set new user default permission details in the catalog.
     * </ul>
     * </dt>
     */
    public void setDefaultPrincipalPermission(String principal, Permission defaultPermission) throws InternalException, AuthorizationException, InvalidArgumentException;

    /**
     * Sets the global default permissions. If a user does not have its own default permissions,
     * these global permissions are used for new entries if no permission is specified.
     *
     * For detailed description of the default permimssions semantics see
     * <A href="../../../../../../overview-summary.html#defaultPermission">Virtual Directory Tree of Permissions</A>.
     * 
     * @param defaultPermission - permission object to be used while no permissions are specified and user in a context has not defaults set
     * 
     * @throws AuthorizationException   No access right to update the default permissions.
     * @throws InvalidArgumentException The permission settings are invalid.
     * @throws InternalException
     *
     * <dt>
     * <br><b>Semantic description of the method:</b>
     * <br>The following steps are performed:
     * <ul>
     * <li> Check the validity of permission settings;
     * <br> if it is invalid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
     * <li> Check WRITE permission on default global permission entry;
     * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
     * <li> Update global default permission details in the catalog.
     * </ul>
     * </dt>
     */
    public void setDefaultGlobalPermission(Permission defaultPermission) throws InternalException, AuthorizationException, InvalidArgumentException;

    /**
     * Returns the {SURL,GUID} pair for a given SURL.
     * 
     * @return the {SURL,GUID} pair for a given SURL.
     * 
     * @throws AuthorizationException   No access right to list the GUID for a given SURL.
     * @throws NotExistsException       The SURL does not exist.
     * @throws InvalidArgumentException Invalid SURL.
     * @throws InternalException
     * 
     * <dt>
     * <br><b>Semantic description of the method:</b>
     * <br>The following steps are performed for each entry:
     * <ul>
     * <li> Check the format of the SURL;
     * <br> if it is invalid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
     * <li> Check if SURL exist;
     * <br> if it does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}.
     * <li> Check LIST permission on the corresponding GUID;
     * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
     * <li> Query GUID for the surl from the catalog and add it to the list to be returned.
     * </ul>
     * </dt>
     */
    public StringPair[] getGuidForSurl(String[] surls) throws InternalException, AuthorizationException, NotExistsException, InvalidArgumentException;

    /**
     * Returns the master SURL for a given GUID.
     * 
     * @param guids the list of GUIDs to get the master SURL for
     * @return the Master SURL of a given GUID, as String Pairs. First string is the GUID, 
     * second the master SURL.
     * 
     * @throws AuthorizationException   No access right to list SURLs for a given GUID.
     * @throws NotExistsException       The GUID does not exist.
     * @throws InvalidArgumentException Invalid GUID.
     * @throws InternalException
     *
     * <dt>
     * <br><b>Semantic description of the method:</b>
     * <br>The following steps are performed for each entry:
     * <ul>
     * <li> Check the format of the GUID;
     * <br> if it is invalid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
     * <li> Check if GUID exists;
     * <br> if it does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}.
     * <li> Check LIST permission on GUID;
     * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
     * <li> Check if replica exists;
     * <br> if no replica exists for the GUID, throws {@link org.glite.data.catalog.service.NotExistsException}.
     * <br> Query for master replica and add it to the list to be returned.
     * </ul>
     * </dt>
     */
    public StringPair[] getMasterReplica(String[] guids) throws InternalException, AuthorizationException, NotExistsException, InvalidArgumentException;

    /**
     * Returns list {GUID,SURL} pairs for given GUIDs.
     * 
     * @param guids - GUID strings we want surls for
     * @return list of {@link org.glite.data.catalog.service.StringPair StringPair} objects. 
     * The first string is the GUID, the second the SURL corresponding to that GUID.
     * If the GUID has more than one SURL, it will be repeated.
     * 
     * @throws AuthorizationException   No access right to list SURLs for a given GUID.
     * @throws NotExistsException       The GUID does not exist.
     * @throws InvalidArgumentException Invalid GUID.
     * @throws InternalException
     *
     * <dt>
     * <br><b>Semantic description of the method:</b>
     * <br>The following steps are performed for each entry:
     * <ul>
     * <li> Check the format of the GUID;
     * <br> if it is invalid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
     * <li> Check if GUID exists;
     * <br> if it does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}.
     * <li> Check LIST permission on GUID;
     * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
     * <li> Query SURLs from the catalog;
     * <br> if no SURL exists for the GUID, throws {@link org.glite.data.catalog.service.NotExistsException}.
     * <li> Add all of the replicas to the list to be returned;
     * </ul>
     * </dt>
     */
    public StringPair[] listSurlsByGuid(String[] guids) throws InternalException, AuthorizationException, NotExistsException, InvalidArgumentException;

    /**
     * Returns list of all <b>valid</b> {@link org.glite.data.catalog.service.RCEntry RCEntry} for given
     * list of GUIDs.
     * 
     * @param guids - list of GUIDs we want valid RCEntries for
     * @return  {@link org.glite.data.catalog.service.RCEntry RCEntry} array, only valid SURLs.
     * 
     * @throws AuthorizationException   No access right to list SURLs for a given GUID.
     * @throws NotExistsException       The GUID does not exist.
     * @throws InvalidArgumentException Invalid format of GUID. 
     * @throws InternalException
     *
     * <dt>
     * <br><b>Semantic description of the method:</b>
     * <br>The following steps are performed for each entry:
     * <ul>
     * <li> Check the format of the GUID;
     * <br> if it is invalid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
     * <li> Check if GUID exists;
     * <br> if it does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}.
     * <li> Check LIST permission on GUID;
     * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
     * <li> Add permission to RCEntry if 'withPermissions' flag is set to true;
     * <li> Add all the replica data from the catalog and add them to the RCEntry list to be returned;
     * </ul>
     * </dt>
     */
    public RCEntry[] listReplicasByGuid(String[] guids, boolean withPermissions) throws InternalException, AuthorizationException, NotExistsException, InvalidArgumentException;

    /**
     * Convenience method to check the existence of a GUID in the catalog.
	 *
     * @param guids  the list of GUIDs to check
     * @return true or false depending whether the GUID exists or not
     * 
     * @throws AuthorizationException   No access right to list the catalog.
     * @throws InvalidArgumentException Invalid format of GUID. 
     * @throws InternalException
	 *
	 * <dt>
	 * <br><b>Semantic description of the method:</b>
	 * <br>The following steps are performed for each entry:
	 * <ul>
     * <li> Check the format of the GUID;
     * <br> if it is invalid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
     * <li> Check if GUID exists and check LIST permission on GUID;
	 * <br> if it exists and permission is granted add true to the list to be returned.
	 * <br> if it does not exist or permission is not granted add false to the list to be returned.
	 * </ul>
	 * </dt>
     */ 
    public StringBoolean[] hasGuid(String[] guids) throws InternalException, InvalidArgumentException, AuthorizationException;

    /**
     * Gets list of GUIDStat {see @GUIDStat} objects for given GUID.
     * 
     * @param guids GUID we want stats for
     * @return GUIDStat object
     * 
     * @throws AuthorizationException   No access right get status information for a given GUID.
	 * @throws InvalidArgumentException Invalid format of GUID.
     * @throws NotExistsException       The GUID does not exist.
     * @throws InternalException
     *
     * <dt>
     * <br><b>Semantic description of the method:</b>
     * <br>The following steps are performed for each entry:
     * <ul>
     * <li> Check the format of the GUID;
     * <br> if it is invalid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
     * <li> Check if GUID exists;
     * <br> if it does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}.
     * <li> Check LIST permission on GUID;
     * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
     * <li> Fill up the GUIDStat object with GUID stats and add it to the list to be returned.
 	 * </ul>
     */
    public GUIDStat[] getGuidStat(String[] guids) throws InternalException, AuthorizationException, NotExistsException,InvalidArgumentException;

    /**
     * Gets list of SURLStat objects for given SURLs
     * 
     * @param surls list of SURLs  we want stats for
     * @return SURLEntry list
	 *
     * @throws InvalidArgumentException Invalid format of SURL
     * @throws AuthorizationException   No access right to list the GUID for a given SURL.
     * @throws NotExistsException       Either the GUID does not exist or the given SURL is not a registered replica of the GUID.
     * @throws InternalException
	 *
	 * <dt>
	 * <br><b>Semantic description of the method:</b>
	 * <br>The following steps are performed for each entry:
	 * <ul>
     * <li> Check the format of the SURL;
     * <br> if it is invalid, throws {@link org.glite.data.catalog.service.InvalidArgumentException}.
	 * <li> Check if SURL exists;
	 * <br> if it does not exist, throws {@link org.glite.data.catalog.service.NotExistsException}.
	 * <li> Check LIST permission on corresponding GUID;
	 * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
	 * <br> Query stat details of SURL from the catalog and add it to the list to be returned;
	 * </ul>
	 * </dt>
     */
    public SURLEntry[] getSurlStat(String[] surls) throws InternalException, AuthorizationException, NotExistsException, InvalidArgumentException;
    
    /**
     * Get the default permissions for user. Defaults to be used for newly created GUIDs
     * where the GUIDStat object does not contain any permissions. If the user
     * does not have is default user permissions set, the global permissions are
     * used. If the user cannot be detected, default global permissions are used.
     *
     *  For detailed description of the default permimssions semantics 
     * see <A href="../../../../../../overview-summary.html#defaultPermission">Virtual Directory Tree of Permissions</A> 
     * @return defaultPermission - permission object to be used while no permissions are specified
     * @see #getDefaultGlobalPermission
     * 
     * @throws AuthorizationException   No access right to get the default user permissions.
	 * @throws InternalException
	 *
	 * <dt>
	 * <br><b>Semantic description of the method:</b>
	 * <br>The following steps are performed:
	 * <ul>
	 * <li> Check READ permission on default user permission entry;
	 * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
	 * <li> Query user default permission details from the catalog and return it.
	 * </ul>
	 * </dt>
     */
    public Permission getDefaultPrincipalPermission(String principal) throws InternalException, AuthorizationException, InvalidArgumentException;

    /**
     * Gets the global default permissions. If a user does not have its own default permissions,
     * these global permissions are used for new entries if no permission is specified.
     *
     *  For detailed description of the default permimssions semantics see
     * <A href="../../../../../../overview-summary.html#defaultPermission">Virtual Directory Tree of Permissions</A>
     *
     * @return defaultPermission - permission object to be used while no permissions are specified and user in a context has not defaults set
     * 
     * @throws AuthorizationException   No access right to get the default permissions.
	 * @throws InternalException
	 *
	 * <dt>
	 * <br><b>Semantic description of the method:</b>
	 * <br>The following steps are performed:
	 * <ul>
	 * <li> Check READ permission on default global permission entry;
	 * <br> if permission is not granted, throws {@link org.glite.data.catalog.service.AuthorizationException}.
	 * <li> Query global default permission details from the catalog and return it.
	 * </ul>
	 * </dt>
     */
    public Permission getDefaultGlobalPermission() throws InternalException, AuthorizationException;
}

